#ifndef _DAMLDB_INT_STRING_MAP_HH
#define _DAMLDB_INT_STRING_MAP_HH

#if defined (sun)
#  pragma ident "@(#)$Id: IntStringMap.hh,v 1.2 2002/04/09 02:04:46 neves Exp $"
#endif

#include "damldb/Portability.hh"

#include <cstdio>
#include <cstdlib>
#include <db.h>
#include <iostream>
#include <stdexcept>
#include <string>

#ifdef CCXX_NAMESPACES
namespace damldb {
USING(std);
#endif

  class IntStringMap
  {
  public:
    typedef int		      key_type;
    typedef string	      data_type;
    typedef string            mapped_type;
    typedef string            value_type;
    typedef value_type*       pointer;
    typedef const value_type* const_pointer;
    typedef value_type&       reference;
    typedef const value_type& const_reference;
    typedef size_t            size_type;
    typedef ptrdiff_t         difference_type;
                               
  public:
    IntStringMap(const string& file, const int open_mode)
      : db_(0), file_name_(file)
    {
      create(open_mode);
      open(open_mode);
    }

    ~IntStringMap()
    {
      if (db_ != 0)
	{
	  db_->close(db_, 0);
	  db_ = 0;
	}
    }

    void sync()
    {
      int err = db_->sync(db_, 0);
      if (err != 0)
	{
	  THROW(runtime_error(db_strerror(err)));
	}
    }

    const_reference find(const key_type& x, value_type& result)
    {
      DBT key;
      DBT data;

      memset(& key, 0, sizeof(key));
      memset(& data, 0, sizeof(data));
      key.data = const_cast<int*>(&x);
      key.size = sizeof(x);

      int err = db_->get(db_, NULL, &key, &data, 0);
      if (err != 0)
	{
	  THROW(runtime_error(db_strerror(err)));
	}

      result = static_cast<char*>(data.data);

      return result;
    }

    void insert(const key_type& k, const value_type& v)
    {
      DBT key;
      DBT data;

      memset(& key, 0, sizeof(key));
      memset(& data, 0, sizeof(data));
      key.data = const_cast<int*>(&k);
      key.size = sizeof(k);

      data.data = const_cast<char*>(v.c_str());
      data.size = v.size()+1;

      int err = db_->put(db_, NULL, & key, & data, 0);
      if (err != 0)
	{
	  THROW(runtime_error(db_strerror(err)));
	}
    }
  
    const_reference find_or_insert(const key_type& k, value_type& v)
    {
      try
	{
	  return find(k, v);
	}
      catch(...)
	{
	  insert(k,v);
	}

      return v;
    }

  private:

    IntStringMap(const IntStringMap&);

    void create(const int open_mode)
    {
      int err = db_create(&db_, NULL, 0);
      if (err != 0)
	{
	  THROW(runtime_error(db_strerror(err)));
	}
    }

    void open(const int open_mode)
    {
      DBTYPE type = DB_BTREE;
      u_int32_t flags = DB_CREATE;
      int mode = 0;

      int err = db_->open(db_, file_name_.c_str(), NULL, type, flags, mode);
      if (err != 0)
	{
	  THROW(runtime_error(db_strerror(err)));
	}
    }

    u_int32_t desired_access(const int open_mode) const
    {
      u_int32_t retVal = DB_RDONLY;

      if ((open_mode & ios::in) && !(open_mode & ios::out)) {
	retVal = DB_RDONLY;
      }

      if (!(open_mode & ios::in) && (open_mode & ios::out)) {
	retVal = DB_CREATE;
      }
      return retVal;
    }

  private:

    DB* db_;
    string file_name_;
  };

#ifdef CCXX_NAMESPACES
} // namespace damldb
#endif

#endif //!_DAMLDB_INT_STRING_MAP_HH

// Emacs
// Local Variables:
// mode: c++
// c-basic-offset: 2
// End:
