#ifndef _DAMLDB_MALLOC_ALLOCATOR_HH
#define _DAMLDB_MALLOC_ALLOCATOR_HH

#if defined (sun)
#  pragma ident "@(#)$Id: MallocAllocator.hh,v 1.2 2002/04/09 02:04:46 neves Exp $"
#endif

#include "damldb/Portability.hh"

template <class T> class MallocAllocator
{
 public:
  typedef T                 value_type;
  typedef value_type*       pointer;
  typedef const value_type* const_pointer;
  typedef value_type&       reference;
  typedef const value_type& const_reference;
  typedef size_t            size_type;
  typedef ptrdiff_t         difference_type;
                           
  template <class U> 
    struct rebind { typedef MallocAllocator<U> other; };

  MallocAllocator() {}
  MallocAllocator(const MallocAllocator&) {}
  template <class U> 
    MallocAllocator(const MallocAllocator<U>&) {}
  ~MallocAllocator() {}

  pointer address(reference x) const { return &x; }
  const_pointer address(const_reference x) const { 
    return x;
  }

  pointer allocate(size_type n, const_pointer = 0) {
    void* p = malloc(n * sizeof(T));
    if (!p)
      THROW(bad_alloc());
    return static_cast<pointer>(p);
  }

  void deallocate(pointer p, size_type) { free(p); }

  size_type max_size() const { 
    return static_cast<size_type>(-1) / sizeof(T);
  }

  void construct(pointer p, const value_type& x) { 
    new(p) value_type(x); 
  }
  void destroy(pointer p) { p->~value_type(); }

 private:
  void operator=(const MallocAllocator&);
};

template<> class MallocAllocator<void>
{
  typedef void        value_type;
  typedef void*       pointer;
  typedef const void* const_pointer;

  template <class U> 
    struct rebind { typedef MallocAllocator<U> other; };
};


template <class T>
inline bool operator==(const MallocAllocator<T>&, 
		       const MallocAllocator<T>&) {
  return true;
}

template <class T>
inline bool operator!=(const MallocAllocator<T>&, 
		       const MallocAllocator<T>&) {
  return false;
}

#endif /*! _DAMLDB_MALLOC_ALLOCATOR_HH */

// Emacs
// Local Variables:
// mode: c++
// c-basic-offset: 2
// End:
