
#ifndef _DAMLDB_RECORDDB_HH
#define _DAMLDB_RECORDDB_HH

#if defined (sun)
#  pragma ident "@(#)$Id: RecordDB.hh,v 1.2 2002/04/09 02:04:46 neves Exp $"
#endif

#include "damldb/Portability.hh"

#include <memory>

#ifdef CCXX_NAMESPACES
namespace damldb {
#endif

template <class T, class Allocator = allocator<T> >
class RecordDB : private Allocator
{
public:
  typedef T value_type;
                               
  typedef typename Allocator::reference	reference;
  typedef typename Allocator::const_reference	const_reference;
  typedef typename Allocator::size_type	size_type;
  typedef typename Allocator::difference_type difference_type;
  typedef typename Allocator::pointer		iterator;
  typedef typename Allocator::const_pointer	const_iterator;
  typedef Allocator allocator_type;

public:
  RecordDB(const size_type n, const Allocator& a = Allocator())
    : Allocator(a), _first(0), _last(0)
  {
    a.set_close_file_on_destroy(false);
    _first = Allocator::allocate(n);
    _last  = _first + Allocator::get_record_count();
    _end_of_storage = Allocator::end_of_storage();
  }

  RecordDB(const RecordDB&);

  virtual ~RecordDB() {}

public:
  allocator_type get_allocator() const {
    return static_cast<const Allocator&>(*this);
  }

  iterator begin()
  { return _first; }

  iterator end()
  { return _last; }

  const_iterator begin() const
  { return _first; }

  const_iterator end() const
  { return _last; }

  size_type size() const
  { return size_type(end() - begin()); }

  size_type max_size() const
  { return size_type(-1) / sizeof(value_type); }

  size_type capacity() const
  { return size_type(_end_of_storage - begin()); }

  bool empty() const
  { return begin() == end(); }

  reference operator[](size_type __n)
  { return *(begin() + __n); }

  const_reference operator[](size_type __n) const
  { return *(begin() + __n); }

  RecordDB& operator=(const RecordDB&);

  void push_back(const T& x) {
    if (_last != _end_of_storage) {
      construct(_last, x);
      ++_last;
      Allocator::set_record_count(size());
    }
    else
      _extend_array(end(), x);
  }

  void push_back() {
    if (_last != _end_of_storage) {
      construct(_last);
      ++_last;
      Allocator::set_record_count(size());
    }
    else
      _extend_array(end());
  }

  void pop_back() {
    --_last;
    Allocator::set_record_count(size());
    destroy(_last);
  }

private:

  void _extend_array(iterator p, const T& x) {
    const size_type old_size = size();
    const size_type len = (old_size != 0) ? 2 * old_size : 1000;
    _first = Allocator::reallocate(len);
    _last = _first + old_size;
    _end_of_storage = Allocator::end_of_storage();

    construct(_last, x);
    ++_last;
    Allocator::set_record_count(size());
  }

  void _extend_array(iterator p) {
    const size_type old_size = size();
    const size_type len = (old_size != 0) ? 2 * old_size : 1000;
    _first = Allocator::reallocate(len);
    _last = _first + old_size;
    _end_of_storage = Allocator::end_of_storage();

    construct(_last);
    ++_last;
    Allocator::set_record_count(size());
  }

private:
  typename Allocator::pointer _first;
  typename Allocator::pointer _last;
  typename Allocator::pointer _end_of_storage;
};

#ifdef CCXX_NAMESPACES
} // namespace damldb
#endif

#endif //!_DAMLDB_RECORDDB_HH

// Emacs
// Local Variables:
// mode: c++
// c-basic-offset: 2
// End:
