
#ifndef _DAMLDB_STRING_INT_MAP_HH
#define _DAMLDB_STRING_INT_MAP_HH

#if defined (sun)
#  pragma ident "@(#)$Id: StringIntMap.hh,v 1.2 2002/04/09 02:04:46 neves Exp $"
#endif

#include "damldb/Portability.hh"

#include <cstdio>
#include <cstdlib>
#include <db.h>
#include <iostream>
#include <stdexcept>
#include <string>

#ifdef CCXX_NAMESPACES
namespace damldb {
#endif

  class StringIntMap
  {
  public:
    typedef string	      key_type;
    typedef int               data_type;
    typedef int               mapped_type;
    typedef int               value_type;
    typedef value_type*       pointer;
    typedef const value_type* const_pointer;
    typedef value_type&       reference;
    typedef const value_type& const_reference;
    typedef size_t            size_type;
    typedef ptrdiff_t         difference_type;
                               
  public:
    StringIntMap(const string& file, const int open_mode)
      : db_(0), file_name_(file)
    {
      create();
      open(open_mode);
    }

    ~StringIntMap()
    {
      if (db_ != 0)
	{
	  db_->close(db_, 0);
	  db_ = 0;
	}
    }

    void sync()
    {
      int err = db_->sync(db_, 0);
      if (err != 0)
	{
	  THROW(runtime_error(db_strerror(err)));
	}
    }

    const_reference find(const key_type& x, value_type& result)
    {
      DBT key;
      DBT data;
      int err;

      memset(& key, 0, sizeof(key));
      memset(& data, 0, sizeof(data));
      key.data = const_cast<char*>(x.c_str());
      key.size = x.size();

      err = db_->get(db_, NULL, &key, &data, 0);

      if (err != 0)
	{
	  THROW(runtime_error(db_strerror(err)));
	}

      result = *static_cast<pointer>(data.data);
      return result;
    }

    void insert(const key_type& k, const value_type& v)
    {
      DBT key;
      DBT data;

      memset(& key, 0, sizeof(key));
      memset(& data, 0, sizeof(data));
      key.data = const_cast<char*>(k.c_str());
      key.size = k.size();

      data.data = const_cast<int*>(&v);
      data.size = sizeof(v);

      int err = db_->put(db_, NULL, & key, & data, 0);
      if (err != 0)
	{
	  THROW(runtime_error(db_strerror(err)));
	}
    }
  
    const_reference find_or_insert(const key_type& k, value_type& v)
    {
      try
	{
	  return find(k, v);
	}
      catch(...)
	{
	  insert(k,v);
	}

      return v;
    }

  private:

    StringIntMap(const StringIntMap&);

    void create()
    {
      int err = db_create(&db_, NULL, 0);
      if (err != 0)
	{
	  THROW(runtime_error(db_strerror(err)));
	}
    }

    void open(const int open_mode)
    {
      DBTYPE type = DB_BTREE;
      u_int32_t flags = DB_CREATE;
      int mode = 0;

      int err = db_->open(db_, file_name_.c_str(), NULL, type, flags, mode);
      if (err != 0)
	{
	  THROW(runtime_error(db_strerror(err)));
	}
    }

  private:

    DB* db_;
    string file_name_;
  };

#ifdef CCXX_NAMESPACES
} // namespace damldb
#endif

#endif //!_DAMLDB_STRING_INT_MAP_HH

// Emacs
// Local Variables:
// mode: c++
// c-basic-offset: 2
// End:
