// $Id: GenerateInterface.java,v 1.1 2002/04/05 04:21:52 neves Exp $


/**
 * generate a skeleton implementation for the specified Java interface
 */
class GenerateInterface
{
    /**
     * clear the specified flag if it's set  
     */
    static int clear(int flags, int flag)
    {
	if ((flags & flag) != 0)
	    {
		flags ^= flag;	// XOR
	    }
	return flags;
    }

    static void generateMethods(Class cl, java.io.PrintWriter stream)
    {
	java.lang.reflect.Method[] methods = cl.getDeclaredMethods();

	for (int i = 0; i < methods.length; i++)
	    {
		java.lang.reflect.Method method = methods[i];
		String methodName = method.getName();
		int modifiers = method.getModifiers();
		modifiers = clear(modifiers, java.lang.reflect.Modifier.ABSTRACT);
		Class returns = method.getReturnType();

		stream.println();
		stream.print("    " + java.lang.reflect.Modifier.toString(modifiers) + " " + returns.getName() + " " + methodName + "(");
		
		Class parameters[] = method.getParameterTypes();
		for (int k = 0; k < parameters.length; k++)
		    {
			Class parameter = parameters[k];
			if (k > 0)
			    {
				stream.print(", ");
			    }
			stream.print(parameters[k].getName() + " p" + k);
		    }
		stream.println(")");
		
		Class exceptions[] = method.getExceptionTypes();
		if (exceptions.length > 0)
		    {
			stream.print("        throws ");
			for (int j = 0; j < exceptions.length; j++)
			    {
				if (j > 0)
				    {
					stream.println(", ");
					stream.print("               ");
				    }
				stream.print(exceptions[j].getName());
			    }
			stream.println();
		    }
		stream.println("    {");
		if (returns != null)
		    {
			stream.print("        " + returns.getName() + " retval = ");
			if (returns.isPrimitive())
			    {
				if (returns.equals(Boolean.TYPE))
				    stream.print("false");
				else
				    stream.print("0");
			    }
			else
			    stream.print("null"); 
			stream.println(";");
		    }
		stream.println("        unimplemented(\"" + methodName + "\");");
		if (returns != null)
		    {
			stream.println("        return retval;");
		    }
		stream.println("    }");
	    }

	// recurse
	Class interfaces[] = cl.getInterfaces();
	for (int m = 0; m < interfaces.length; m++)
	    {
		generateMethods(interfaces[m], stream);
	    }
    }

    static void usage()
    {
	System.err.println("Usage:  InterfaceName implementationClassName");
	System.exit(1);
    }

    public static void main(String args[])
	throws Exception
    {
	// argument parsing
	if (args.length != 2)
	    usage();
	String interfaceName = args[0];
	String implementationName = args[1];

	Class cl = Class.forName(interfaceName);

	java.io.PrintWriter stream = new java.io.PrintWriter(new java.io.FileWriter(implementationName + ".java"));

	stream.println("// $" + "Id$");
	stream.println();
	stream.println();
	stream.println("class " + implementationName);
	stream.println("    implements " + interfaceName);
	stream.println("{");
	stream.println("    static void unimplemented(String name)");
	stream.println("    {");
	stream.println("        System.err.println(\"" + implementationName + ".\" + name + \" unimplemented\");");
	stream.println("    }");

	generateMethods(cl, stream);

	stream.println("}");
	stream.close();
    }
}
