
#ifndef lint
static const char revid[] =
	"@(#)$Id: DAMLDB.cpp,v 1.2 2002/04/09 21:59:51 neves Exp $";
#endif

#include "damldb/DAMLDB.hh"
#include "damldb/DBFactory.hh"
#include "damldb/DBFactoryImpl.hh"
#include "damldb/IDamlDB.hh"
#include "damldb/OS.hh"
#include "damldb/Singleton.hh"

#include <iostream>
#include <memory>

#if defined(_MSC_VER)
#define WIN32_LEAN_AND_MEAN   // Exclude rarely-used stuff from Windows headers
#include <windows.h>

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}
#endif


#ifdef CCXX_NAMESPACES
namespace damldb {
USING(std);
#endif

  typedef CSingleton<DAMLDB> tDAMLDB;
  tDAMLDB::InstPtr tDAMLDB::sm_ptr;

  // CONSTANTS

  const string kDAMLDB_DEFAULT_DATA_DIRECTORY = string(DEFAULT_DATA_DIRECTORY);
  const char*  kDAMLDB_DATA_DIR_ENV_VAR = "DAMLDB_DATA_DIR";

  // CLASS VARIABLES

  DAMLDB*	DAMLDB::instance_  = 0;
  string	DAMLDB::data_dir_  = kDAMLDB_DEFAULT_DATA_DIRECTORY;
  int		DAMLDB::open_mode_ = iodb::in | iodb::out | iodb::nocreate;

  //////////////////////////////////////////////////////////////////////

  DAMLDB::DAMLDB()
  {
    char* envValue = OS::getenv(kDAMLDB_DATA_DIR_ENV_VAR);

    const string dataDirOverride((envValue == NULL) ? "" : envValue);

    if (kDAMLDB_DEFAULT_DATA_DIRECTORY == DAMLDB::getDataDirectory() 
			&& !dataDirOverride.empty())
      {
	DAMLDB::setDataDirectory(dataDirOverride);
      }

    auto_ptr<DBFactory> factory(new DBFactoryImpl());
    db = factory->newDatabase(DAMLDB::getDataDirectory(), DAMLDB::get_open_mode());
  }

  //////////////////////////////////////////////////////////////////////

  DAMLDB::~DAMLDB()
  {
    delete db;
  }

  //////////////////////////////////////////////////////////////////////

  DAMLDB * DAMLDB::getInstance()
  {
    return tDAMLDB::Instance();
  }

  //////////////////////////////////////////////////////////////////////

  const int DAMLDB::get_open_mode()
  {
    return open_mode_;
  }

  //////////////////////////////////////////////////////////////////////

  void DAMLDB::set_open_mode(const int open_mode)
  {
    open_mode_ = open_mode;
  }

  //////////////////////////////////////////////////////////////////////

  const string& DAMLDB::getDataDirectory()
  {
    return data_dir_;
  }

  //////////////////////////////////////////////////////////////////////

  void DAMLDB::setDataDirectory(const string& dataDirectory)
  {
    data_dir_ = dataDirectory;
  }

#ifdef CCXX_NAMESPACES
} //namespace damldb
#endif

// Emacs
// Local Variables:
// mode: c++
// c-basic-offset: 2
// End:
