
#ifndef lint
static const char revid[] = 
	"@(#)$Id: Trace.cpp,v 1.2 2002/04/09 21:59:51 neves Exp $";
#endif

#include <cstdarg>
#include <cstdio>
#include <cstdlib>
#include <iostream>

#include "damldb/Trace.hh"

#if defined(HAVE_LIBLOG4CPP)
#include "log4cpp/Appender.hh"
#include "log4cpp/BasicLayout.hh"
#include "log4cpp/Category.hh"
#include "log4cpp/Layout.hh"
#include "log4cpp/NDC.hh"
#include "log4cpp/OstreamAppender.hh"
#include "log4cpp/Priority.hh"
#include "log4cpp/SimpleConfigurator.hh"
#endif

#ifdef CCXX_NAMESPACES
namespace damldb {
USING(std);
#if defined(HAVE_LIBLOG4CPP)
USING(log4cpp);
#endif

#endif

  Trace*    Trace::instance_ = 0;

  Trace::Trace()
  {
#if defined(HAVE_LIBLOG4CPP)
    try
      {
	char* datadir = getenv("DAMLDB_DATA_DIR");
	string initFileName((datadir == NULL) ?
			    "./log4cpp.init" :
			    string(datadir) + "/log4cpp.init");
	SimpleConfigurator::configure(initFileName);
      }
    catch(ConfigureFailure& f)
      {
	cout << "Configure Problem " << f.what() << endl;
	exit(EXIT_FAILURE);
      }

    root_ = &Category::getRoot();
    root_->info("Trace Initialized");
#endif
  }

  Trace* Trace::getInstance()
  {
    if (instance_ == 0)
      {
	instance_ = new Trace();
      }

    return instance_;
  }

  void Trace::error(const char* stringFormat, ...)
  {
    char buf[1024];
    va_list va;
    va_start(va,stringFormat);
    vsprintf(buf, stringFormat, va);
#if defined(HAVE_LIBLOG4CPP)
    root_->error(buf);
#else
    cout << "ERROR: " << buf << endl;
#endif
    va_end(va);
  }

  void Trace::debug(const char* stringFormat, ...)
  {
    char buf[1024];
    va_list va;
    va_start(va,stringFormat);
    vsprintf(buf, stringFormat, va);
#if defined(HAVE_LIBLOG4CPP)
    root_->debug(buf);
#else
    cout << "DEBUG: " << buf << endl;
#endif
    va_end(va);
  }

  void Trace::info(const char* stringFormat, ...)
  {
    char buf[1024];
    va_list va;
    va_start(va,stringFormat);
    vsprintf(buf, stringFormat, va);
#if defined(HAVE_LIBLOG4CPP)
    root_->info(buf);
#else
    cout << "INFO: " << buf << endl;
#endif
    va_end(va);
  }

  void Trace::warn(const char* stringFormat, ...)
  {
    char buf[1024];
    va_list va;
    va_start(va,stringFormat);
    vsprintf(buf, stringFormat, va);
#if defined(HAVE_LIBLOG4CPP)
    root_->warn(buf);
#else
    cout << "WARN: " << buf << endl;
#endif
    va_end(va);
  }

  void Trace::ndc_push(const char* msg)
  {
#if defined(HAVE_LIBLOG4CPP)
    NDC::push(msg);
#endif
  }

  void Trace::ndc_pop()
  {
#if defined(HAVE_LIBLOG4CPP)
    NDC::pop();
#endif
  }

#ifdef CCXX_NAMESPACES
} // namespace damldb
#endif

// Emacs
// Local Variables:
// mode: c++
// c-basic-offset: 2
// End:

