// Java Native Interface (JNI) to damldb

#ifndef lint
static const char revid[] = 
    "$Id: java_DAMLDB.cpp,v 1.4 2002/04/10 21:21:49 neves Exp $";
#endif // not lint

#if defined(__CYGWIN__)
typedef long long int __int64;
#endif

#include "damldb/DAMLDB.hh"
#include "damldb/IDamlDB.hh"
#include "damldb/Statement.hh"

#include "org_daml_damldb_DAMLDB.h"

#include <jni.h>
#include <iostream>
#include <string>

USING(damldb);

JNIEXPORT void JNICALL Java_org_daml_damldb_DAMLDB_init
  (JNIEnv* env, jclass)
{
  try {
    IDamlDB& db = *DAMLDB::getInstance()->db;
  } catch (...) {
    jclass newExcCls = env->FindClass("java/lang/IllegalStateException");
    if (newExcCls != 0) {
      env->ThrowNew(newExcCls, "thrown from C++ code: Check for database existence !!!");
    }
  }
}

JNIEXPORT jlong JNICALL Java_org_daml_damldb_DAMLDB_size
  (JNIEnv* env, jclass)
{
  jlong retval = -1;

  try {
    IDamlDB& db = *DAMLDB::getInstance()->db;
    retval = db.get_statement_count();
  } catch (...) {
    jclass newExcCls = env->FindClass("java/lang/IllegalStateException");
    if (newExcCls != 0) {
      env->ThrowNew(newExcCls, "thrown from C++ code: java_DAMLDB.cpp");
    }
  }
  return retval;
}

JNIEXPORT jlong JNICALL Java_org_daml_damldb_DAMLDB_firstSubject
  (JNIEnv* env, jclass, jlong resource)
{
  jlong retval = NULL_STATEMENT_ID;
  try {
    IDamlDB& db = *DAMLDB::getInstance()->db;
    retval = db.get_resource(resource).subjectFirst;
  } catch(...) {
    jclass newExcCls = env->FindClass("java/lang/IllegalArgumentException");
    if (newExcCls != 0) {
      env->ThrowNew(newExcCls, "thrown from C++ code: java_DAMLDB.cpp");
    }
  }
  return retval;
}

JNIEXPORT jlong JNICALL Java_org_daml_damldb_DAMLDB_firstPredicate
  (JNIEnv* env, jclass, jlong resource)
{
  jlong retval = NULL_STATEMENT_ID;
  try {
    IDamlDB& db = *DAMLDB::getInstance()->db;
    retval = db.get_resource(resource).predicateFirst;
  } catch(...) {
    jclass newExcCls = env->FindClass("java/lang/IllegalArgumentException");
    if (newExcCls != 0) {
      env->ThrowNew(newExcCls, "thrown from C++ code: java_DAMLDB.cpp");
    }
  }
  return retval;
}

JNIEXPORT jlong JNICALL Java_org_daml_damldb_DAMLDB_firstObject
  (JNIEnv* env, jclass, jlong resource)
{
  jlong retval = NULL_STATEMENT_ID;
  try {
    IDamlDB& db = *DAMLDB::getInstance()->db;
    retval = db.get_resource(resource).objectFirst;
  } catch(...) {
    jclass newExcCls = env->FindClass("java/lang/IllegalArgumentException");
    if (newExcCls != 0) {
      env->ThrowNew(newExcCls, "thrown from C++ code: java_DAMLDB.cpp");
    }
  }
  return retval;
}

JNIEXPORT jlong JNICALL Java_org_daml_damldb_DAMLDB_subject
  (JNIEnv* env, jclass, jlong statement)
{
  jlong retval = NULL_STATEMENT_ID;
  try {
    IDamlDB& db = *DAMLDB::getInstance()->db;
    retval = db.get_statement(statement).subject;
  } catch(...) {
    jclass newExcCls = env->FindClass("java/lang/IllegalArgumentException");
    if (newExcCls != 0) {
      env->ThrowNew(newExcCls, "thrown from C++ code: java_DAMLDB.cpp");
    }
  }
  return retval;
}

JNIEXPORT jlong JNICALL Java_org_daml_damldb_DAMLDB_predicate
  (JNIEnv* env, jclass, jlong statement)
{
  jlong retval = NULL_STATEMENT_ID;
  try {
    IDamlDB& db = *DAMLDB::getInstance()->db;
    retval = db.get_statement(statement).predicate;
  } catch(...) {
    jclass newExcCls = env->FindClass("java/lang/IllegalArgumentException");
    if (newExcCls != 0) {
      env->ThrowNew(newExcCls, "thrown from C++ code: java_DAMLDB.cpp");
    }
  }
  return retval;
}

JNIEXPORT jlong JNICALL Java_org_daml_damldb_DAMLDB_object
  (JNIEnv* env, jclass, jlong statement)
{
  jlong retval = NULL_STATEMENT_ID;
  try {
    IDamlDB& db = *DAMLDB::getInstance()->db;
    retval = db.get_statement(statement).object;
  } catch(...) {
    jclass newExcCls = env->FindClass("java/lang/IllegalArgumentException");
    if (newExcCls != 0) {
      env->ThrowNew(newExcCls, "thrown from C++ code: java_DAMLDB.cpp");
    }
  }
  return retval;
}

JNIEXPORT jlong JNICALL Java_org_daml_damldb_DAMLDB_source
  (JNIEnv* env, jclass, jlong statement)
{
  jlong retval = NULL_STATEMENT_ID;
  try {
    IDamlDB& db = *DAMLDB::getInstance()->db;
    retval = db.get_statement(statement).source;
  } catch(...) {
    jclass newExcCls = env->FindClass("java/lang/IllegalArgumentException");
    if (newExcCls != 0) {
      env->ThrowNew(newExcCls, "thrown from C++ code: java_DAMLDB.cpp");
    }
  }
  return retval;
}

JNIEXPORT jint JNICALL Java_org_daml_damldb_DAMLDB_statementFlags
  (JNIEnv* env, jclass, jlong statement)
{
  jlong retval = NULL_STATEMENT_ID;
  try {
    IDamlDB& db = *DAMLDB::getInstance()->db;
    retval = db.get_statement(statement).flags;
  } catch(...) {
    jclass newExcCls = env->FindClass("java/lang/IllegalArgumentException");
    if (newExcCls != 0) {
      env->ThrowNew(newExcCls, "thrown from C++ code: java_DAMLDB.cpp");
    }
  }
  return retval;
}

JNIEXPORT jlong JNICALL Java_org_daml_damldb_DAMLDB_nextSubject
  (JNIEnv* env, jclass, jlong statement)
{
  jlong retval = NULL_STATEMENT_ID;
  try {
    IDamlDB& db = *DAMLDB::getInstance()->db;
    retval = db.get_statement(statement).subjectNext;
  } catch(...) {
    jclass newExcCls = env->FindClass("java/lang/IllegalArgumentException");
    if (newExcCls != 0) {
      env->ThrowNew(newExcCls, "thrown from C++ code: java_DAMLDB.cpp");
    }
  }
  return retval;
}

JNIEXPORT jlong JNICALL Java_org_daml_damldb_DAMLDB_nextPredicate
  (JNIEnv* env, jclass, jlong statement)
{
  jlong retval = NULL_STATEMENT_ID;
  try {
    IDamlDB& db = *DAMLDB::getInstance()->db;
    retval = db.get_statement(statement).predicateNext;
  } catch(...) {
    jclass newExcCls = env->FindClass("java/lang/IllegalArgumentException");
    if (newExcCls != 0) {
      env->ThrowNew(newExcCls, "thrown from C++ code: java_DAMLDB.cpp");
    }
  }
  return retval;
}

JNIEXPORT jlong JNICALL Java_org_daml_damldb_DAMLDB_nextObject
  (JNIEnv* env, jclass, jlong statement)
{
  jlong retval = NULL_STATEMENT_ID;
  try {
    IDamlDB& db = *DAMLDB::getInstance()->db;
    retval = db.get_statement(statement).objectNext;
  } catch(...) {
    jclass newExcCls = env->FindClass("java/lang/IllegalArgumentException");
    if (newExcCls != 0) {
      env->ThrowNew(newExcCls, "thrown from C++ code: java_DAMLDB.cpp");
    }
  }
  return retval;
}

JNIEXPORT jlong JNICALL Java_org_daml_damldb_DAMLDB_toLong
  (JNIEnv *env, jclass, jstring juri)
{
  jlong retval;
  const char *uri = env->GetStringUTFChars(juri, 0);
  IDamlDB& db = *DAMLDB::getInstance()->db;
  retval = db.uri2resource(uri);
  env->ReleaseStringUTFChars(juri, uri);
  return retval;
}

JNIEXPORT jstring JNICALL Java_org_daml_damldb_DAMLDB_toString
  (JNIEnv *env, jclass, jlong index)
{
  jstring retval = 0;
  try {
    IDamlDB& db = *DAMLDB::getInstance()->db;
    string result;
    const string& buf = db.resource2uri(index, result);
    retval = env->NewStringUTF(buf.c_str());
  } catch(...) {}
  return retval;
}

JNIEXPORT jlong JNICALL Java_org_daml_damldb_DAMLDB_addStatement
  (JNIEnv *, jclass, jlong sourceId, jlong subjectId, jlong predicateId, jlong objectId)
{
  jlong retval = NULL_STATEMENT_ID;
  try {
    IDamlDB& db = *DAMLDB::getInstance()->db;
    retval = db.get_statement_count();
    db.add_statement(sourceId, subjectId, predicateId, objectId);
  } catch(...) {}
  return retval;
}

JNIEXPORT jlong JNICALL Java_org_daml_damldb_DAMLDB_addResource
  (JNIEnv *env, jclass, jstring juri)
{
  jlong retval;
  const char *uri = env->GetStringUTFChars(juri, 0);
  IDamlDB& db = *DAMLDB::getInstance()->db;
  retval = db.uri2resource(uri);
  env->ReleaseStringUTFChars(juri, uri);
  return retval;
}

// Emacs
// Local Variables:
// mode: c++
// c-basic-offset: 2
// End:
